use strict;

package ASCOPE::Class::Text::Encoded;
use base qw (ASCOPE::Class::Text);

$ASCOPE::Class::Text::Encoded::VERSION = '1.0';

use Memoize;

# Note: you might think the code below is slower than just
# handing off the value of ->raw() to &_encode whole hog.
# Apparently not...

# Benchmark: timing 50000000 iterations of one, two...
#       one:  4 wallclock secs ( 4.45 usr +  0.01 sys =  4.45 CPU) @ 11228070.18/s (n=50000000)
#       two:  5 wallclock secs ( 5.12 usr + -0.02 sys =  5.09 CPU) @ 9815950.92/s (n=50000000)

use overload q("") => sub {
  join(" ",map {
    $_ = &_encode($_)
  } split(" ",shift->raw()));
};

my %entity_map = (
		  nbsp   => "&#160;",
		  iexcl  => "&#161;",
		  cent   => "&#162;",
		  pound  => "&#163;",
		  curren => "&#164;",
		  yen    => "&#165;",
		  brvbar => "&#166;",
		  sect   => "&#167;",
		  uml    => "&#168;",
		  copy   => "&#169;",
		  ordf   => "&#170;",
		  laquo  => "&#171;",
		  not    => "&#172;",
		  shy    => "&#173;",
		  reg    => "&#174;",
		  macr   => "&#175;",
		  deg    => "&#176;",
		  plusmn => "&#177;",
		  sup2   => "&#178;",
		  sup3   => "&#179;",
		  acute  => "&#180;",
		  micro  => "&#181;",
		  para   => "&#182;",
		  middot => "&#183;",
		  cedil  => "&#184;",
		  sup1   => "&#185;",
		  ordm   => "&#186;",
		  raquo  => "&#187;",
		  frac14 => "&#188;",
		  frac12 => "&#189;",
		  frac34 => "&#190;",
		  iquest => "&#191;",
		  Agrave => "&#192;",
		  Aacute => "&#193;",
		  Acirc  => "&#194;",
		  Atilde => "&#195;",
		  Auml   => "&#196;",
		  Aring  => "&#197;",
		  AElig  => "&#198;",
		  Ccedil => "&#199;",
		  Egrave => "&#200;",
		  Eacute => "&#201;",
		  Ecirc  => "&#202;",
		  Euml   => "&#203;",
		  Igrave => "&#204;",
		  Iacute => "&#205;",
		  Icirc  => "&#206;",
		  Iuml   => "&#207;",
		  ETH    => "&#208;",
		  Ntilde => "&#209;",
		  Ograve => "&#210;",
		  Oacute => "&#211;",
		  Ocirc  => "&#212;",
		  Otilde => "&#213;",
		  Ouml   => "&#214;",
		  times  => "&#215;",
		  Oslash => "&#216;",
		  Ugrave => "&#217;",
		  Uacute => "&#218;",
		  Ucirc  => "&#219;",
		  Uuml   => "&#220;",
		  Yacute => "&#221;",
		  THORN  => "&#222;",
		  szlig  => "&#223;",
		  agrave => "&#224;",
		  aacute => "&#225;",
		  acirc  => "&#226;",
		  atilde => "&#227;",
		  auml   => "&#228;",
		  aring  => "&#229;",
		  aelig  => "&#230;",
		  ccedil => "&#231;",
		  egrave => "&#232;",
		  eacute => "&#233;",
		  ecirc  => "&#234;",
		  euml   => "&#235;",
		  igrave => "&#236;",
		  iacute => "&#237;",
		  icirc  => "&#238;",
		  iuml   => "&#239;",
		  eth    => "&#240;",
		  ntilde => "&#241;",
		  ograve => "&#242;",
		  oacute => "&#243;",
		  ocirc  => "&#244;",
		  otilde => "&#245;",
		  ouml   => "&#246;",
		  divide => "&#247;",
		  oslash => "&#248;",
		  ugrave => "&#249;",
		  uacute => "&#250;",
		  ucirc  => "&#251;",
		  uuml   => "&#252;",
		  yacute => "&#253;",
		  thorn  => "&#254;",
		  yuml   => "&#255;");

my $regexp_entities = join('|', keys %entity_map);

sub _encode {
  my $text = shift;

  $text =~ s/&(?!(#[0-9]+|#x[0-9a-fA-F]+|\w+);)/&amp;/gm;
  $text =~ s/&($regexp_entities);/$entity_map{$1}/gm;
  $text =~ s/</&lt;/gm;
  $text =~ s/>/&gt;/gm;

  return $text;
}

END { memoize("_encode"); }

return 1;

__END__

=head1 NAME

ASCOPE::Class::Text::CDATA - Aaron's class thingy for XML encoded text widgets.

=head1 SYNOPSIS

 use ASCOPE::Class::Text::Encoded;
 my $t = ASCOPE::Class::Text::Encoded->new("hello, Montr&eacute;al");

 print $t."\n";

 # prints:
 hello Montr&#233;al

=head1 DESCRIPTION

Aaron's class thingy for XML encoded text widgets.

Overloads the object's I<""> operator to return the value
of the text passed to the object constructor with the various bits and
pieces encoded to keep and XML parser happy.

=head1 PACKAGE METHODS

=head2 __PACKAGE__->new($text)

Returns an object. Woot!

=head1 OBJECT METHODS

=head2 $obj->raw()

Get the raw text passed to the object constructor.

Returns a string.

=head1 VERSION

1.0

=head1 DATE

$Date: 2003/03/12 02:33:31 $

=head1 AUTHOR

Aaron Straup Cope

=head1 SEE ALSO

L<ASCOPE::Class::Text>

=head1 LICENSE

Copyright (c) 2003 Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under
the same terms as Perl itself.

=cut

